//-----------------------------------------------------------------------------
// CursorVar.c
//   Defines the routines specific to the cursor type.
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// Cursor variable type
//-----------------------------------------------------------------------------
typedef struct {
  Variable_HEAD
  OCIStmt **i_Data;
} udt_CursorVar;


//-----------------------------------------------------------------------------
// Declaration of cursor variable functions.
//-----------------------------------------------------------------------------
static int CursorVar_Initialize(udt_CursorVar*);
static int CursorVar_Bind(udt_CursorVar*, udt_Cursor*, PyObject*);
static int CursorVar_SetValue(udt_CursorVar*, unsigned, PyObject*);


//-----------------------------------------------------------------------------
// variable method declaration
//-----------------------------------------------------------------------------
static udt_VariableMethods g_CursorVarMethods = {
  (InitializeProc) CursorVar_Initialize,
  (FinalizeProc) NULL,
  (DefineProc) NULL,
  (BindProc) CursorVar_Bind,
  (SetValueProc) CursorVar_SetValue,
  (GetValueProc) NULL
};


//-----------------------------------------------------------------------------
// Python type declaration
//-----------------------------------------------------------------------------
static PyTypeObject g_CursorVarType = {
  PyObject_HEAD_INIT(NULL) 0,		// header initialization
  "CursorVar",				// type name string
  sizeof(udt_CursorVar),		// basic size (memory allocated)
  sizeof(OCIStmt**) + sizeof(sb2),	// item size (variable object)
  (destructor) Variable_Free,		// deallocation routine
  0,					// print function
  0,					// getattr function
  0,					// setattr function
  0,					// compare function
  0,					// repr function
  0,					// as number function
  0,					// as sequence function
  0,					// as mapping function
  0,					// hash function
  0,					// call function
  0,					// as string function
  (getattrofunc) Variable_GetAttr,	// getattro function
  0,					// setattro function
  0,					// as buffer function
  Py_TPFLAGS_DEFAULT,			// flags
  0					// documentation string
};


//-----------------------------------------------------------------------------
// CursorVar_Initialize()
//   Initialize the variable.
//-----------------------------------------------------------------------------
static int CursorVar_Initialize(
  udt_CursorVar *a_Var)			// variable to initialize
{
  a_Var->i_Data =
      (OCIStmt**) &(a_Var->i_Indicator[a_Var->i_AllocatedElements]);
  return 0;
}


//-----------------------------------------------------------------------------
// CursorVar_Bind()
//   Bind the variable to the given cursor.
//-----------------------------------------------------------------------------
static int CursorVar_Bind(
  udt_CursorVar *a_Var,			// variable to bind
  udt_Cursor *a_Cursor,			// cursor to perform bind for
  PyObject *a_Name)			// name of bind variable
{
  sword v_Status;

  v_Status = OCIBindByName(a_Cursor->i_Handle, &a_Var->i_BindHandle,
      a_Cursor->i_Connection->i_ErrorHandle, PyString_AS_STRING(a_Name),
      PyString_GET_SIZE(a_Name), a_Var->i_Data, sizeof(OCIStmt*), SQLT_RSET,
      a_Var->i_Indicator, 0, 0, 0, 0, OCI_DEFAULT);
  return Connection_CheckForError(a_Cursor->i_Connection, v_Status);
}


//-----------------------------------------------------------------------------
// CursorVar_SetValue()
//   Set the value of the variable.
//-----------------------------------------------------------------------------
static int CursorVar_SetValue(
  udt_CursorVar *a_Var,			// variable to set value for
  unsigned a_Pos,			// array position to set
  PyObject *a_Value)			// value to set
{
  udt_Cursor *v_Value;

  if (a_Value->ob_type != &g_CursorType) {
    PyErr_SetString(PyExc_TypeError, "expecting cursor");
    return -1;
  }

  v_Value = (udt_Cursor *) a_Value;
  a_Var->i_Data[a_Pos] = v_Value->i_Handle;
  v_Value->i_IsQuery = -1;
  return 0;
}

