//-----------------------------------------------------------------------------
// LobVar.c
//   Defines the routines for handling LOB variables.
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// LOB type
//-----------------------------------------------------------------------------
typedef struct {
  Variable_HEAD
  OCILobLocator **i_Data;
} udt_LobVar;

#include "ExternalLobVar.c"

//-----------------------------------------------------------------------------
// Declaration of LOB variable functions.
//-----------------------------------------------------------------------------
static int LobVar_Initialize(udt_LobVar*);
static void LobVar_Finalize(udt_LobVar*);
static int LobVar_Define(udt_LobVar*, udt_Cursor*, int, OCIParam*);
static PyObject *LobVar_GetValue(udt_LobVar*, unsigned);


//-----------------------------------------------------------------------------
// variable method declaration
//-----------------------------------------------------------------------------
static udt_VariableMethods g_LobVarMethods = {
  (InitializeProc) LobVar_Initialize,
  (FinalizeProc) LobVar_Finalize,
  (DefineProc) LobVar_Define,
  (BindProc) NULL,
  (SetValueProc) NULL,
  (GetValueProc) LobVar_GetValue
};


//-----------------------------------------------------------------------------
// Python type declaration
//-----------------------------------------------------------------------------
static PyTypeObject g_LobVarType = {
  PyObject_HEAD_INIT(NULL) 0,		// header initialization
  "LobVar",				// type name string
  sizeof(udt_LobVar),			// basic size (memory allocated)
  sizeof(OCILobLocator**) + sizeof(sb2),
					// item size (variable object)
  (destructor) Variable_Free,		// deallocation routine
  0,					// print function
  0,					// getattr function
  0,					// setattr function
  0,					// compare function
  0,					// repr function
  0,					// as number function
  0,					// as sequence function
  0,					// as mapping function
  0,					// hash function
  0,					// call function
  0,					// as string function
  (getattrofunc) Variable_GetAttr,      // getattro function
  0,					// setattro function
  0,					// as buffer function
  Py_TPFLAGS_DEFAULT,			// flags
  0					// documentation string
};


//-----------------------------------------------------------------------------
// LobVar_Initialize()
//   Initialize the variable.
//-----------------------------------------------------------------------------
static int LobVar_Initialize(
  udt_LobVar *a_Var)			// variable to initialize
{
  sword v_Status;
  int i;

  // initialize members
  a_Var->i_Data =
      (OCILobLocator**) &(a_Var->i_Indicator[a_Var->i_AllocatedElements]);

  // initialize the LOB locators
  for (i = 0; i < a_Var->i_AllocatedElements; i++) {
    v_Status = OCIDescriptorAlloc(a_Var->i_Connection->i_EnvHandle,
        (dvoid**) &a_Var->i_Data[i], OCI_DTYPE_LOB, 0, 0);
    if (Connection_CheckForError(a_Var->i_Connection, v_Status) < 0)
      return -1;
  }

  return 0;
}


//-----------------------------------------------------------------------------
// LobVar_Finalize()
//   Prepare for variable destruction.
//-----------------------------------------------------------------------------
static void LobVar_Finalize(
  udt_LobVar *a_Var)			// variable to free
{
  int i;

  for (i = 0; i < a_Var->i_AllocatedElements; i++) {
    if (a_Var->i_Data[i])
      OCIDescriptorFree(a_Var->i_Data[i], OCI_DTYPE_LOB);
  }
}


//-----------------------------------------------------------------------------
// LobVar_Define()
//   Perform the OCI define for the variable.
//-----------------------------------------------------------------------------
static int LobVar_Define(
  udt_LobVar *a_Var,			// variable to initialize
  udt_Cursor *a_Cursor,			// cursor associated with variable
  int a_Position,			// position to define
  OCIParam *a_Param)			// parameter handle
{
  sword v_Status;

  v_Status = OCIDefineByPos(a_Cursor->i_Handle, &a_Var->i_DefineHandle,
      a_Cursor->i_Connection->i_ErrorHandle, a_Position, a_Var->i_Data,
      sizeof(OCILobLocator*), a_Var->i_DataType, a_Var->i_Indicator, 0, 0,
      OCI_DEFAULT);
  return Connection_CheckForError(a_Cursor->i_Connection, v_Status);
}


//-----------------------------------------------------------------------------
// LobVar_GetValue()
//   Returns the value stored at the given array position.
//-----------------------------------------------------------------------------
static PyObject *LobVar_GetValue(
  udt_LobVar *a_Var,			// variable to determine value for
  unsigned a_Pos)			// array position
{
  return ExternalLobVar_New(a_Var, a_Pos);
}

