#------------------------------------------------------------------------------
# Setup.py
#   Setup module for the module cx_Oracle. For Win32, use the option
# --compiler=mingw32 for compilation.
#------------------------------------------------------------------------------

import os, sys
from distutils.core import setup
from distutils.extension import Extension

# define some variables
v_OracleHome = os.environ["ORACLE_HOME"]
if sys.platform == "win32":
  v_Includes = [os.path.join(v_OracleHome, "oci", "include")]
  v_LibDirs = ["/Tools/Lib"]
  v_Libs = ["oci"]
  v_DataFiles = [ ("cx_Oracle-doc", \
        ["LICENSE.txt", "README.txt", "HISTORY.txt"]) ]
elif sys.platform == "cygwin":
  v_Includes = ["/usr/include", "rdbms/demo", "rdbms/public", \
      "network/public", "oci", "include"]
  v_LibDirs = ["bin", "lib"]
  for i in range(len(v_Includes)):
    v_Includes[i] = os.path.join(v_OracleHome, v_Includes[i])
  for i in range(len(v_LibDirs)):
    v_LibDirs[i] = os.path.join(v_OracleHome, v_LibDirs[i])
  v_Libs = ["oci"]
  v_DataFiles = [ ("cx_Oracle-doc", \
        ["LICENSE.txt", "README.txt", "HISTORY.txt"]) ]
else:
  v_Includes = ["rdbms/demo", "rdbms/public", "network/public"]
  for i in range(len(v_Includes)):
    v_Includes[i] = os.path.join(v_OracleHome, v_Includes[i])
  v_LibDirs = [os.path.join(v_OracleHome, "lib")]
  v_Libs = ["clntsh"]
  v_DataFiles = None

# setup extra compile args
v_ExtraCompileArgs = None
if sys.platform == "aix4":
  v_ExtraCompileArgs = ["-qcpluscmt"]
elif sys.platform == "cygwin":
  v_ExtraCompileArgs = ["-mno-cygwin"]
  
# setup the extension
v_Extension = Extension(
    name = "cx_Oracle",
    include_dirs = v_Includes,
    libraries = v_Libs,
    library_dirs = v_LibDirs,
    extra_compile_args = v_ExtraCompileArgs,
    extra_link_args = ["-s"],
    sources = ["cx_Oracle.c"])

# perform the setup
setup(
    name = "cx_Oracle",
    version = "2.5a",
    description = "Python interface to Oracle",
    licence = "See LICENSE.txt",
    data_files = v_DataFiles,
    long_description = \
        "Python interface to Oracle conforming to the Python DB API 2.0 " + \
        "specification.\n" + \
        "See http://www.python.org/topics/database/DatabaseAPI-2.0.html.",
    author = "Anthony Tuininga",
    author_email = "anthony@computronix.com",
    url = "http://www.computronix.com/utilities",
    ext_modules = [v_Extension])

